#----------------------------------------------------------------
# Main Window
#----------------------------------------------------------------

proc mainWindow {} { global s c w l

    # Menubar
    #
    frame .menu -bd 1 -relief flat
    pack .menu -side top -fill x
    mb .menu.f File .menu.f.m
    mb .menu.m Mode .menu.m.m
    mb .menu.h Help .menu.h.m
    label .menu.l -text "TkScan v$c(version)" -width 15 -pady 0 \
          -fg red -font -*-times-bold-r-*--14-140-*
    pack .menu.f .menu.m .menu.l -side left
    pack .menu.h -side right
    menu .menu.f.m -tearoff false
         .menu.f.m add command -label Preference -command preferenceWindow
         .menu.f.m add command -label "Set current as Default" \
                               -command "savecur Default"
         .menu.f.m add command -label "Save current config" \
                               -command "savesettings"
         .menu.f.m add separator
         .menu.f.m add command -label Quit -command do_exit
    menu .menu.m.m -tearoff false
    foreach i $c(mode_quick_set) {
        set qmodename [lindex $i 0]
        .menu.m.m add command -command "setquickmode $qmodename" \
                              -label $qmodename
    }
    menu .menu.h.m -tearoff false
         .menu.h.m add command -label About  -command aboutWindow
         .menu.h.m add command -label Readme -command readmeWindow
         .menu.h.m add command -label Guide  -command usageWindow

    # Separator - not indispensible, just for good looking.
    canvas .sep1  -relief sunken -height 0 -borderwidth 1
    pack .sep1 -side top -fill x

    # Left frame
    #
    frame .left -bd 5 -relief flat
    pack .left -side left -anchor nw -fill y

    # Preview image canvas
    set w(prev) .left.preview
    canvas $w(prev) -relief sunken
    pack $w(prev) -side top
    bind $w(prev) <ButtonPress-1>	"settop		%x %y"
    bind $w(prev) <B1-Motion>		"setbottom	%x %y"
    bind $w(prev) <ButtonPress-2>	"registrect	%x %y"
    bind $w(prev) <ButtonPress-3>	"deleterect	%x %y"
    bind $w(prev) <Double-Button-3>	"deleteallrect"

    # Right frame
    #
    frame .right -bd 5 -relief flat
    pack .right -side left -expand yes -fill both

    # OUTPUT
    set w(ou) .right.output
    set w(outputto) $w(ou).u.m
    frame $w(ou) -bd 1 -relief groove
    frame $w(ou).u
    pack $w(ou).u -side top -fill x
    rl $w(ou).u.l "Output to"
    menubutton $w(outputto) -menu $w(ou).u.m.m \
               -anchor w -width 18 -padx 6 -pady 2 -relief ridge -fg $l(c1)
    pack $w(ou).u.l $w(ou).u.m -side left -fill x
    menu $w(ou).u.m.m -tearoff false
    foreach i $c(outputtoList) { set label [lindex $i 0]
        $w(ou).u.m.m add command -label $label -command "setoutputto $label"
    }

    # Scanning mode selection
    set w(mo) .right.mode
    frame $w(mo) -bd 1 -relief groove
    # MODE
    frame $w(mo).m
    pack $w(mo).m -side top -anchor nw -fill x
    rl $w(mo).m.label "Mode"
    menubutton $w(mo).m.mode -menu $w(mo).m.mode.m \
               -anchor w -width 18 -padx 6 -pady 2 -relief ridge -fg $l(c1)
    pack $w(mo).m.label $w(mo).m.mode -side left
    menu $w(mo).m.mode.m -tearoff false
    foreach i $s(modeList) { set modename [lindex $i 0]
        $w(mo).m.mode.m add command -label $modename \
                                    -command "setmode $modename"
    }
    # RESOLUTION
    frame $w(mo).r
    pack $w(mo).r -side top -anchor nw
    rl $w(mo).r.label "Resolution"
    entry $w(mo).r.entry -textvariable s(resolution) \
          -justify right -relief sunken -width 4 -font $l(f1) -fg $l(c1)
    scale $w(mo).r.scale -variable s(resolution) -command setresolution \
          -from [lindex $s(resolutionRange) 0] \
          -to [lindex $s(resolutionRange) 1] \
          -resolution [lindex $s(resolutionRange) 2] \
          -length 100 -orient horizontal -showvalue false
    pack $w(mo).r.label $w(mo).r.entry $w(mo).r.scale -side left
    # OPTIONS
    frame $w(mo).e
    pack $w(mo).e -side top -anchor nw
    rl $w(mo).e.label "Options"
    button $w(mo).e.button -text "Open Manager >>" \
           -command optionWindow \
           -anchor e -width 19 -relief ridge -padx 0 -pady 0
    set w(optionbutton) $w(mo).e.button
    pack $w(mo).e.label $w(mo).e.button -side left

    # SCANNING GEOMETRY DISPLAY
    set w(ge) .right.geometry
    frame $w(ge) -bd 1 -relief groove
    canvas $w(ge).xy -width 210 -height 80 -bd 1 -relief flat
    pack $w(ge).xy -fill x
    set w(ft) $w(ge).xy
    set finetune_buttons {
        { tx- {  8 17 } {  8 33 } {  0 25 } }
        { tx+ { 13 17 } { 13 33 } { 21 25 } }
        { bx- { 38 17 } { 38 33 } { 30 25 } }
        { bx+ { 43 17 } { 43 33 } { 51 25 } }
        { ty- { 17  8 } { 33  8 } { 25  0 } }
        { ty+ { 17 13 } { 33 13 } { 25 21 } }
        { by- { 17 38 } { 33 38 } { 25 30 } }
        { by+ { 17 43 } { 33 43 } { 25 51 } }
    }
    set xs 86
    set ys 21
    $w(ft) create rectangle [expr 10+$xs] [expr 10+$ys] \
                            [expr 40+$xs] [expr 40+$ys] -outline blue
    foreach i $finetune_buttons {
        set btag [lindex $i 0]
        for {set j 1} {$j<=3} {incr j} {
            set x$j [expr [lindex [lindex $i $j] 0] +$xs]
            set y$j [expr [lindex [lindex $i $j] 1] +$ys]
        }
        $w(ft) create polygon $x1 $y1 $x2 $y2 $x3 $y3 -fill black -tag $btag
        $w(ft) bind $btag <Any-Enter> "$w(ft) itemconfig current -fill red"
        $w(ft) bind $btag <Any-Leave> "$w(ft) itemconfig current -fill black"
        $w(ft) bind $btag <ButtonPress> "set s(xx) 0; finetune $btag"
        $w(ft) bind $btag <ButtonRelease> "set s(xx) 1"
    }
    # Coordinates
    $w(ft) create text [expr 5+$xs] [expr 15+$ys] -tag ul -anchor se \
                       -fill $l(c1) -font $l(f1)
    $w(ft) create text [expr 45+$xs] [expr 35+$ys] -tag lr -anchor nw \
                       -fill $l(c1) -font $l(f1)
    # Label and Buttons
    rl $w(ft).l "Geometry"
    label  $w(ft).u -text "Length Unit"
    menubutton $w(ft).m -text $s(unit) -menu $w(ft).m.m \
                        -width 4 -pady 0 -relief ridge -fg $l(c1)
    menu $w(ft).m.m -tearoff false
    foreach i $s(unitList) { set label [lindex $i 0]
        $w(ft).m.m add command -label $label -command \
                 "setunit $label ; $w(ft).m configure -text $label"
    }
    menubutton $w(ft).p -text "Preset" -menu $w(ft).p.m \
                        -width 6 -pady 0 -relief ridge
    menu $w(ft).p.m -tearoff false
    foreach i $c(size_quick_set) {
        set qpapername [lindex $i 0]
        $w(ft).p.m add command -command "setquickpaper $qpapername" \
                              -label $qpapername
    }
    button $w(ft).r -text Regist -command {registrect $s(tx0) $s(ty0)} \
                    -padx 0 -pady 0 -relief ridge
    button $w(ft).d -text Delete -command {deleterect $s(tx0) $s(ty0)} \
                    -padx 0 -pady 0 -relief ridge
    button $w(ft).c -text Clear  -command deleteallrect \
                    -padx 0 -pady 0  -relief ridge
    $w(ft) create window   0   0 -window $w(ft).l -anchor nw
    $w(ft) create window 102   0 -window $w(ft).u -anchor nw
    $w(ft) create window 177   0 -window $w(ft).m -anchor nw
    $w(ft) create window 163  22 -window $w(ft).p -anchor nw
    $w(ft) create window   0  39 -window $w(ft).r -anchor nw
    $w(ft) create window   0  61 -window $w(ft).d -anchor nw
    $w(ft) create window  46  61 -window $w(ft).c -anchor nw

    # SIZE : Real, Screen, Data
    sizeline $w(ge).rs "Real Size" "mm^2"
    sizeline $w(ge).ss "Screen Size" "pixel^2"
    sizeline $w(ge).ds "Data Size" "KB"

    # BUTTONS
    set w(bt) .right.buttons
    frame $w(bt) -bd 1 -relief flat
    button $w(bt).p -text Preview -command "Preview" -width 5 -pady 4
    button $w(bt).s -text SCAN -command {Scan [timetagscanout]} \
                    -width 10 -pady 4
    button $w(bt).q -text Stop -command "set s(stop) 1" \
                    -width 3 -pady 4 -state disabled
    bind $w(bt).q <Double-Button-1> {
        set s(pid) [getpid $s(command)]
        if {"$s(pid)" != ""} { foreach i $s(pid) { exec kill -9 $i } }
        set s(stop) 0
        set s(runscan) 0
        $w(bt).p configure -state normal
        $w(bt).s configure -state normal
        $w(bt).q configure -state disabled
    }
    pack $w(bt).p $w(bt).s $w(bt).q -side left -expand yes -fill x

    # MESSAGE LINE
    # Scanning progress: percentage in  bar and number
    set w(st) .right.status
    set w(stat_text) $w(st).text
    set w(stat_prog) $w(st).prog
    set w(prog_bar) $w(stat_prog).bar
    set w(prog_perc) $w(stat_prog).percent
    set s(prog_bar_wd) 173
    frame $w(st) -bd 1 -relief groove
    pack $w(st) -side bottom -fill x
    label $w(stat_text) -anchor w -relief flat -font $l(f1)
    frame $w(stat_prog)
    pack $w(stat_text) $w(stat_prog) -side top -fill x
    canvas $w(prog_bar) -width $s(prog_bar_wd) -height 15 -bd 1 -relief ridge
    label $w(prog_perc) -width 5 -padx 1 -pady 0 -anchor e -bd 1 -relief ridge
    pack $w(prog_bar) $w(prog_perc) -side left
    set s(sg) [$w(prog_bar) create rectangle 0 0 0 18 -fill $l(c6)]

    pack $w(ou) $w(mo) $w(ge) $w(bt) -side top -expand yes -fill x
    pack $w(st) -side bottom -expand yes -fill x

}

